/* Programming activity: HashMaps and File I/O (SOLUTION)
 *
 * Part 1: Complete the missing code to read in the username and passwords from
 * the input file and put them into a HashMap. Use the HashMap to allow the user
 * (via standard keyboard input) to try and authenticate.  Sample output:
 * 
 *   % java Login users-passwords.txt
 *   Total usernames: 9572
 *   
 *   Enter username: bob
 *   Username not found!
 *   
 *   % java Login users-passwords.txt
 *   Total usernames: 9572
 *   
 *   Enter username: ramie
 *   Enter password: blah
 *   Invalid password!
 *   
 *   %java Login users-passwords.txt
 *   Total usernames: 9572
 *   
 *   Enter username: ramie
 *   Enter password: nash
 *   Password correct.
 *   
 * Part 2: Complete the missing code to search for all usernames and passwords
 * that have the given prefix (they start with this text). Sample output:
 *   ...
 *   Enter prefix: bob
 *   
 *   Username starting with prefix:
 *   bobbyd
 *   bobette
 *   bobbie
 *   bobst
 *   bobbyed
 *   bobbies
 *   
 *   Passwords starting with prefix:
 *   bobdylan
 *   bobobo
 *   bobby
 *   bob123
 *   bobby
 *
 * Part 3: Complete the missing code to search for all usernames with a given
 * password.  Sample output:
 *   ...
 *   Find usernames with this password: megan
 *   
 *   Username with this password:
 *   acarmley
 *   lsalvatori
 *   aoberholzer
 */

import java.io.*;
import java.util.*;

public class Login
{

    public static void main(String[] args)
    {
        // Program takes a single command line argument specifying the text file
        // contain usernames and passwords.
        if (args.length < 1)
        {
            System.out.println("Login <username/password filename>");
            return;
        }
        String filename = args[0];
        
        /////////////////////////////////////////////////////////////////////////////////
        // Part 1
        
        // Open up the username/password file for reading
        Scanner scanner = null;
        try
        {
            scanner = new Scanner(new File(filename));
        }
        catch (FileNotFoundException e)
        {
            System.out.println("Failed to open file: " + filename);
            return;
        }
        
        // Create a HashMap that has usernames as the key and the password as the value
        HashMap<String, String> usernameToPassword = new HashMap<String, String>();
                
        // Read the usernames and passwords from the file into the HashMap
        while (scanner.hasNext())
        {
            String username = scanner.next();
            String password = scanner.next();
        
            usernameToPassword.put(username, password);
        }                       
        scanner.close();
        
        // Print out the total count of keys in the HashMap (i.e. the number of users)
        System.out.println("Total usernames: " + usernameToPassword.size());
        System.out.println();
        
        System.out.print("Enter username: ");
        String username = StdIn.readString();
        
        // Check that the HashMap has this username as a key
        if (usernameToPassword.containsKey(username))
        {
            System.out.print("Enter password: ");
            String password = StdIn.readString();

            // Given the username key, check the value in the HashMap to see if it matches the entered password
            if (usernameToPassword.get(username).equals(password))
            {
                System.out.println("Password correct.");
            }
            else
            {
                System.out.println("Invalid password!");
            }
        }
        else
        {
            System.out.println("Username not found!");
        }
        
        /////////////////////////////////////////////////////////////////////////////////
        // Part 2        
        
        System.out.println();
        System.out.print("Enter prefix: ");
        String prefix = StdIn.readString();        
        System.out.println();
        System.out.println("Username starting with prefix:");
                
        for (String key : usernameToPassword.keySet())
        {
            if (key.startsWith(prefix))
                System.out.println(key);
        }

        System.out.println();
        System.out.println("Passwords starting with prefix:");
        for (String value : usernameToPassword.values())
        {
            if (value.startsWith(prefix))
                System.out.println(value);
        }
     
        /////////////////////////////////////////////////////////////////////////////////
        // Part 3        
        
        System.out.println();
        System.out.print("Find usernames with this password: ");
        String password = StdIn.readString();
        System.out.println();
        System.out.println("Username with this password: ");
        
        for (Map.Entry<String, String> entry : usernameToPassword.entrySet())
        {
            if (entry.getValue().equals(password))
                System.out.println(entry.getKey());
        }
               
    }

}
